import type { Disposer, Entry, Fetcher, FetchOptions, GetOptions, HasOptions, PeekOptions, SetOptions, SizeCalculator } from 'lru-cache';
import * as LRUCache from 'lru-cache';
/**
 * LRU cache service.
 *
 * Allows you to specify <K, V> (key, value) type parameters.
 */
export declare class LruCache<K = any, V = any> {
    private readonly _cache;
    constructor(_cache: LRUCache<K, V>);
    /**
     * The maximum number of entries that remain in the cache (assuming no TTL pruning or explicit deletions).
     */
    get max(): number;
    /**
     * Maximum size of all entries added to the cache.
     */
    get maxSize(): number;
    /**
     * Maximum size of an entry added to the cache.
     */
    get maxEntrySize(): number;
    /**
     * Function used to calculate the size of stored entries.
     */
    get sizeCalculation(): SizeCalculator<K, V> | undefined;
    /**
     * Function that is called on items when they are dropped from the cache, as `this.dispose(value, key, reason)`.
     */
    get dispose(): Disposer<K, V>;
    /**
     * The same as dispose, but called after the entry is completely removed and the cache is once again in a clean
     * state.
     */
    get disposeAfter(): Disposer<K, V> | null;
    /**
     * Suppresses calling the `dispose()` function if the entry key is still accessible within the cache.
     */
    get noDisposeOnSet(): boolean;
    /**
     * Max time to live for items before they are considered stale.
     */
    get ttl(): number;
    /**
     * Minimum amount of time in ms in which to check for staleness.
     */
    get ttlResolution(): number;
    /**
     * Whether to preemptively remove stale items from the cache.
     */
    get ttlAutopurge(): boolean;
    /**
     * Whether to return the stale value on `get()` as well as deleting it.
     */
    get allowStale(): boolean;
    /**
     * Whether to make each item's age reset to `0` whenever it is retrieved from cache with `get()`, causing it to not
     * expire.
     */
    get updateAgeOnGet(): boolean;
    /**
     * Whether to make stale items to remain in the cache, until they are explicitly deleted with `delete()`.
     */
    get noDeleteOnStaleGet(): boolean;
    /**
     * Function that is used to make background asynchronous fetches.
     *
     * Called with `fetchMethod(key, staleValue, { signal, options, context })`.
     */
    get fetchMethod(): Fetcher<K, V> | null;
    /**
     * The total number of items held in the cache at the current moment.
     */
    get size(): number;
    /**
     * The total size of items in cache when using size tracking.
     */
    get calculatedSize(): number;
    /**
     * Checks if a key is in the cache. Will return `false` if the item is stale, even though it is technically in
     * the cache.
     *
     * Will not update item age unless `updateAgeOnHas` is set in the options or constructor.
     *
     * @param key The key to check.
     * @param options Addition options.
     */
    has(key: K, options?: HasOptions): boolean;
    /**
     * Returns a value from the cache.
     *
     * If the key is not found, `get()` will return `undefined`.
     * This can be confusing when setting values specifically to `undefined`, as in `set(key, undefined)`. Use `has()`
     * to determine whether a key is present in the cache at all.
     */
    get<T = unknown>(key: K, options?: GetOptions): T | undefined;
    /**
     * Like `get()` but doesn't update recency or delete stale items.
     *
     * Returns `undefined` if the item is stale, unless `allowStale` is set either on the cache or in the options object.
     */
    peek<T = unknown>(key: K, options?: PeekOptions): T | undefined;
    /**
     * Adds a value to the cache.
     *
     * The third parameter can be either a TTL number, or options object.
     */
    set(key: K, value: V, ttl?: number): this;
    set(key: K, value: V, options?: SetOptions<K, V>): this;
    /**
     * Deletes a key out of the cache.
     *
     * Returns `true` if the key was deleted, `false` otherwise.
     */
    delete(key: K): boolean;
    /**
     * Clears the cache entirely, throwing away all values.
     */
    clear(): void;
    /**
     * Delete any stale entries.
     *
     * Returns `true` if anything was removed, `false` otherwise.
     */
    purgeStale(): boolean;
    /**
     * Find a value for which the supplied callback function returns a truthy value, similar to `Array.find()`.
     *
     * The function is called as `fn(value, key, cache)`.
     */
    find<T = V>(callbackFn: (value: V, key: K, cache: LRUCache<K, V>) => void | boolean | undefined, options?: GetOptions): T;
    /**
     * Call the supplied function on each item in the cache, in order from most recently used to least recently used.
     *
     * The function is called as `fn(value, key, cache)`.  Does not update age or recenty of use.
     */
    forEach<T = this>(callbackFn: (this: T, value: V, key: K, cache: LRUCache<K, V>) => void, thisArg?: T): void;
    /**
     * The same as `cache.forEach(...)` but items are iterated over in reverse order.
     * (ie, less recently used items are iterated over first.)
     */
    rforEach<T = this>(callbackFn: (this: T, value: V, key: K, cache: LRUCache<K, V>) => void, thisArg?: T): void;
    /**
     * Returns the remaining time before an item expires.
     *
     * Returns `0` if the item is not found in the cache or is already expired.
     */
    getRemainingTTL(key: K): number;
    /**
     * Return a generator yielding the keys in the cache, in order from most recently used to least recently used.
     */
    keys(): Generator<K>;
    /**
     * Inverse order version of `cache.keys()`.
     *
     * Return a generator yielding the keys in the cache, in order from least recently used to most recently used.
     */
    rkeys(): Generator<K>;
    /**
     * Return a generator yielding the values in the cache, in order from most recently used to least recently used.
     */
    values(): Generator<V>;
    /**
     * Inverse order version of `cache.values()`.
     *
     * Return a generator yielding the values in the cache, in order from least recently used to most recently used.
     */
    rvalues(): Generator<V>;
    /**
     * Return a generator yielding `[key, value]` pairs, in order from most recently used to least recently used.
     */
    entries(): Generator<[K, V]>;
    /**
     * Inverse order version of `cache.entries()`.
     *
     * Return a generator yielding `[key, value]` pairs, in order from least recently used to most recently used.
     */
    rentries(): Generator<[K, V]>;
    /**
     * Return an array of `[key, entry]` objects which can be passed to `cache.load()`.
     */
    dump(): Array<[K, Entry<V>]>;
    /**
     * Reset the cache and load in the items in entries in the order listed.
     *
     * Note that the shape of the resulting cache may be different if the same options are not used in both caches.
     */
    load(cacheEntries: ReadonlyArray<[K, Entry<V>]>): void;
    /**
     * Evicts the least recently used item, returning its value or `undefined` if cache is empty.
     */
    pop(): V | undefined;
    fetch<ExpectedValue = V>(key: K, options?: FetchOptions<K, V>): Promise<ExpectedValue | undefined>;
    /**
     * Iterating over the cache itself yields the same results as `cache.entries()`.
     */
    [Symbol.iterator](): Iterator<[K, V]>;
}
//# sourceMappingURL=lru-cache.d.ts.map