"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LruCache = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const LRUCache = require("lru-cache");
const lru_cache_constants_1 = require("../lru-cache.constants");
/**
 * LRU cache service.
 *
 * Allows you to specify <K, V> (key, value) type parameters.
 */
let LruCache = class LruCache {
    constructor(_cache) {
        this._cache = _cache;
    }
    /**
     * The maximum number of entries that remain in the cache (assuming no TTL pruning or explicit deletions).
     */
    get max() {
        return this._cache.max;
    }
    /**
     * Maximum size of all entries added to the cache.
     */
    get maxSize() {
        return this._cache.maxSize;
    }
    /**
     * Maximum size of an entry added to the cache.
     */
    get maxEntrySize() {
        return this._cache.maxEntrySize;
    }
    /**
     * Function used to calculate the size of stored entries.
     */
    get sizeCalculation() {
        return this._cache.sizeCalculation;
    }
    /**
     * Function that is called on items when they are dropped from the cache, as `this.dispose(value, key, reason)`.
     */
    get dispose() {
        return this._cache.dispose;
    }
    /**
     * The same as dispose, but called after the entry is completely removed and the cache is once again in a clean
     * state.
     */
    get disposeAfter() {
        return this._cache.disposeAfter;
    }
    /**
     * Suppresses calling the `dispose()` function if the entry key is still accessible within the cache.
     */
    get noDisposeOnSet() {
        return this._cache.noDisposeOnSet;
    }
    /**
     * Max time to live for items before they are considered stale.
     */
    get ttl() {
        return this._cache.ttl;
    }
    /**
     * Minimum amount of time in ms in which to check for staleness.
     */
    get ttlResolution() {
        return this._cache.ttlResolution;
    }
    /**
     * Whether to preemptively remove stale items from the cache.
     */
    get ttlAutopurge() {
        return this._cache.ttlAutopurge;
    }
    /**
     * Whether to return the stale value on `get()` as well as deleting it.
     */
    get allowStale() {
        return this._cache.allowStale;
    }
    /**
     * Whether to make each item's age reset to `0` whenever it is retrieved from cache with `get()`, causing it to not
     * expire.
     */
    get updateAgeOnGet() {
        return this._cache.updateAgeOnGet;
    }
    /**
     * Whether to make stale items to remain in the cache, until they are explicitly deleted with `delete()`.
     */
    get noDeleteOnStaleGet() {
        return this._cache.noDeleteOnStaleGet;
    }
    /**
     * Function that is used to make background asynchronous fetches.
     *
     * Called with `fetchMethod(key, staleValue, { signal, options, context })`.
     */
    get fetchMethod() {
        return this._cache.fetchMethod;
    }
    /**
     * The total number of items held in the cache at the current moment.
     */
    get size() {
        return this._cache.size;
    }
    /**
     * The total size of items in cache when using size tracking.
     */
    get calculatedSize() {
        return this._cache.calculatedSize;
    }
    /**
     * Checks if a key is in the cache. Will return `false` if the item is stale, even though it is technically in
     * the cache.
     *
     * Will not update item age unless `updateAgeOnHas` is set in the options or constructor.
     *
     * @param key The key to check.
     * @param options Addition options.
     */
    has(key, options) {
        return this._cache.has(key, options);
    }
    /**
     * Returns a value from the cache.
     *
     * If the key is not found, `get()` will return `undefined`.
     * This can be confusing when setting values specifically to `undefined`, as in `set(key, undefined)`. Use `has()`
     * to determine whether a key is present in the cache at all.
     */
    get(key, options) {
        return this._cache.get(key, options);
    }
    /**
     * Like `get()` but doesn't update recency or delete stale items.
     *
     * Returns `undefined` if the item is stale, unless `allowStale` is set either on the cache or in the options object.
     */
    peek(key, options) {
        return this._cache.peek(key, options);
    }
    set(key, value, optionsOrTtl) {
        optionsOrTtl = typeof optionsOrTtl === 'number' ? { ttl: optionsOrTtl } : optionsOrTtl;
        this._cache.set(key, value, optionsOrTtl);
        return this;
    }
    /**
     * Deletes a key out of the cache.
     *
     * Returns `true` if the key was deleted, `false` otherwise.
     */
    delete(key) {
        return this._cache.delete(key);
    }
    /**
     * Clears the cache entirely, throwing away all values.
     */
    clear() {
        return this._cache.clear();
    }
    /**
     * Delete any stale entries.
     *
     * Returns `true` if anything was removed, `false` otherwise.
     */
    purgeStale() {
        return this._cache.purgeStale();
    }
    /**
     * Find a value for which the supplied callback function returns a truthy value, similar to `Array.find()`.
     *
     * The function is called as `fn(value, key, cache)`.
     */
    find(
    // eslint-disable-next-line @typescript-eslint/no-invalid-void-type
    callbackFn, options) {
        return this._cache.find(callbackFn, options);
    }
    /**
     * Call the supplied function on each item in the cache, in order from most recently used to least recently used.
     *
     * The function is called as `fn(value, key, cache)`.  Does not update age or recenty of use.
     */
    forEach(callbackFn, thisArg) {
        return this._cache.forEach(callbackFn, thisArg);
    }
    /**
     * The same as `cache.forEach(...)` but items are iterated over in reverse order.
     * (ie, less recently used items are iterated over first.)
     */
    rforEach(callbackFn, thisArg) {
        return this._cache.rforEach(callbackFn, thisArg);
    }
    /**
     * Returns the remaining time before an item expires.
     *
     * Returns `0` if the item is not found in the cache or is already expired.
     */
    getRemainingTTL(key) {
        return this._cache.getRemainingTTL(key);
    }
    /**
     * Return a generator yielding the keys in the cache, in order from most recently used to least recently used.
     */
    *keys() {
        yield* this._cache.keys();
    }
    /**
     * Inverse order version of `cache.keys()`.
     *
     * Return a generator yielding the keys in the cache, in order from least recently used to most recently used.
     */
    *rkeys() {
        yield* this._cache.rkeys();
    }
    /**
     * Return a generator yielding the values in the cache, in order from most recently used to least recently used.
     */
    *values() {
        yield* this._cache.values();
    }
    /**
     * Inverse order version of `cache.values()`.
     *
     * Return a generator yielding the values in the cache, in order from least recently used to most recently used.
     */
    *rvalues() {
        yield* this._cache.rvalues();
    }
    /**
     * Return a generator yielding `[key, value]` pairs, in order from most recently used to least recently used.
     */
    *entries() {
        yield* this._cache.entries();
    }
    /**
     * Inverse order version of `cache.entries()`.
     *
     * Return a generator yielding `[key, value]` pairs, in order from least recently used to most recently used.
     */
    *rentries() {
        yield* this._cache.rentries();
    }
    /**
     * Return an array of `[key, entry]` objects which can be passed to `cache.load()`.
     */
    dump() {
        return this._cache.dump();
    }
    /**
     * Reset the cache and load in the items in entries in the order listed.
     *
     * Note that the shape of the resulting cache may be different if the same options are not used in both caches.
     */
    load(cacheEntries) {
        return this._cache.load(cacheEntries);
    }
    /**
     * Evicts the least recently used item, returning its value or `undefined` if cache is empty.
     */
    pop() {
        return this._cache.pop();
    }
    fetch(key, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return yield this._cache.fetch(key, options);
        });
    }
    /**
     * Iterating over the cache itself yields the same results as `cache.entries()`.
     */
    [Symbol.iterator]() {
        return this.entries();
    }
};
LruCache = tslib_1.__decorate([
    (0, common_1.Injectable)(),
    tslib_1.__param(0, (0, common_1.Inject)(lru_cache_constants_1.LRU_CACHE)),
    tslib_1.__metadata("design:paramtypes", [LRUCache])
], LruCache);
exports.LruCache = LruCache;
//# sourceMappingURL=lru-cache.js.map